# -*- coding: utf-8 -*-

import sys

from logbook import Logger, StreamHandler, StderrHandler
from logbook.more import ColorizedStderrHandler

from .utils import ExecutionContext


__all__ = [
    "user_log",
]


def user_log_formatter(record, handler):
    return "[{dt}] {level}: {msg}".format(
        dt=ExecutionContext.get_current_dt(),
        level=record.level_name,
        msg=record.message,
    )


# handler = StreamHandler(sys.stdout)
handler = ColorizedStderrHandler()
handler.formatter = user_log_formatter
handler.push_application()


user_log = Logger("user_log")


def user_print(*args, **kwargs):
    sep = kwargs.get("sep", " ")
    end = kwargs.get("end", "")

    message = sep.join(map(str, args)) + end

    user_log.info(message)



# -*- coding: utf-8 -*-

from .fixture import *
from rqalpha import StrategyExecutor
from rqalpha.api import *


def test_import_api():
    from rqalpha.api import (
        order_shares, order_percent, order_lots, order_value,
        order_target_value, order_target_percent, get_order,
        get_open_orders, cancel_order, update_universe,
        instruments, history, is_st_stock,
    )


def test_instrument(trading_params, data_proxy):
    def init(context):
        context.stocks = []

    def handle_bar(context, bar_dict):
        if len(context.stocks) == 0:
            context.stocks.append(instruments("600485.XSHG"))

    executor = StrategyExecutor(
        init=init,
        handle_bar=handle_bar,

        trading_params=trading_params,
        data_proxy=data_proxy,
    )

    perf = executor.execute()

    i = executor.strategy_context.stocks[0]
    assert i.order_book_id == "600485.XSHG"



# -*- coding: utf-8 -*-

from pprint import pprint

import pandas as pd
import numpy as np
from six import iteritems

from rqalpha import StrategyExecutor
from rqalpha.api import order_shares
from rqalpha.trading_params import TradingParams
from .fixture import *


def test_get_trading_dates(data_proxy):
    start_date, end_date = pd.Timestamp("2014-01-04"), pd.Timestamp("2014-01-07")
    trading_cal = data_proxy.get_trading_dates(start_date, end_date)

    assert len(trading_cal) == 2



# -*- coding: utf-8 -*-

from enum import Enum


ORDER_STATUS = Enum("ORDER_STATUS", [
    "OPEN",
    "FILLED",
    "REJECTED",
    "CANCELLED",
])


EVENT_TYPE = Enum("EVENT_TYPE", [
    "DAY_START",
    "HANDLE_BAR",
    "DAY_END",
])


EXECUTION_PHASE = Enum("EXECUTION_PHASE", [
    "INIT",
    "HANDLE_BAR",
    "BEFORE_TRADING",
    "SCHEDULED",
    "FINALIZED",
])


class DAYS_CNT(object):
    DAYS_A_YEAR = 365
    TRADING_DAYS_A_YEAR = 252



# -*- coding: utf-8 -*-

from collections import defaultdict

# TODO make field readonly
# TODO use nametuple to reduce memory


class Position(object):

    def __init__(self):
        self.quantity = 0.          # int	未平仓部分的总股数。
        self.bought_quantity = 0.   # int	该证券的总买入股数，例如：如果你的投资组合并没有任何平安银行的成交，那么平安银行这个股票的仓位就是0.
        self.sold_quantity = 0.     # int	该证券的总卖出股数，例如：如果你的投资组合曾经买入过平安银行股票200股并且卖出过100股，那么这个属性会返回100.
        self.bought_value = 0.      # float	该证券的总买入的价值，等于每一个该证券的买入成交的价格*买入股数的总和。
        self.sold_value = 0.        # float	该证券的总卖出价值，等于每一个该证券的卖出成交的价格*卖出股数的总和。
        # self.total_orders = 0.      # int	该仓位的总订单的次数。
        # self.total_trades = 0.      # int	该仓位的总成交的次数。
        self.sellable = 0.          # int	该仓位可卖出股数。T＋1的市场中sellable = 所有持仓-今日买入的仓位。
        self.average_cost = 0.      # float	获得该持仓的买入均价，计算方法为每次买入的数量做加权平均。
        self.market_value = 0.      # float	获得该持仓的实时市场价值。
        self.value_percent = 0.     # float	获得该持仓的实时市场价值在总投资组合价值中所占比例，取值范围[0, 1]。

    def __repr__(self):
        return "Position({%s})" % self.__dict__


def Positions():
    return defaultdict(Position)



# -*- coding: utf-8 -*-
import numpy as np
import talib
import math

def getExtremem(arrayHighPriceResult, arrayLowPriceResult):
    np_arrayHighPriceResult = np.array(arrayHighPriceResult[:-1])
    np_arrayLowPriceResult = np.array(arrayLowPriceResult[:-1])
    maxResult = np_arrayHighPriceResult.max()
    minResult = np_arrayLowPriceResult.min()
    return [maxResult, minResult]

def getAtrAndUnit(atrArrayResult, atrLengthResult, portfolioValueResult):
    atr = atrArrayResult[atrLengthResult-1]
    unit = math.floor(portfolioValueResult * .01 / atr)
    return [atr, unit]

def getStopPrice(firstOpenPriceResult, units_hold_result, atrResult):
    stopPrice =  firstOpenPriceResult - 2*atrResult + (units_hold_result-1)*0.5*atrResult
    return stopPrice


def init(context):
    context.tradedayNum = 0
    context.unit = 0
    context.atr = 0
    context.tradingSignal = 'start'
    context.preTradingSignal = ''
    context.units_hold_max = 4
    context.units_hold = 0
    context.quantity = 0
    context.max_add = 0
    context.firstOpenPrice = 0
    context.s = 'CSI300.INDX'
    context.openObserveTime = 55;
    context.closeObserveTime = 20;
    context.atrTime = 20;

def handle_bar(context, bar_dict):
    portfolioValue = context.portfolio.portfolio_value
    highPrice = history(context.openObserveTime+1, '1d', 'high')[context.s]
    lowPriceForAtr = history(context.openObserveTime+1, '1d', 'low')[context.s]
    lowPriceForExtremem = history(context.closeObserveTime+1, '1d', 'low')[context.s]
    closePrice = history(context.openObserveTime+2, '1d', 'close')[context.s]
    closePriceForAtr = closePrice[:-1]

    atrArray = talib.ATR(highPrice.values, lowPriceForAtr.values, closePriceForAtr.values, timeperiod=context.atrTime)

    maxx = getExtremem(highPrice.values, lowPriceForExtremem.values)[0]
    minn = getExtremem(highPrice.values, lowPriceForExtremem.values)[1]
    atr = atrArray[-2]


    if (context.tradingSignal != 'start'):
        if (context.units_hold != 0):
            context.max_add += 0.5 * getAtrAndUnit(atrArray, atrArray.size, portfolioValue)[0]
    else:
        context.max_add = bar_dict[context.s].last


    curPosition = context.portfolio.positions[context.s].quantity
    availableCash = context.portfolio.cash
    marketValue = context.portfolio.market_value


    if (curPosition > 0 and bar_dict[context.s].last < getStopPrice(context.firstOpenPrice, context.units_hold, atr)):
        context.tradingSignal = 'stop'
    else:
        if (curPosition > 0 and bar_dict[context.s].last < minn):
            context.tradingSignal = 'exit'
        else:
            if (bar_dict[context.s].last > context.max_add and context.units_hold != 0 and context.units_hold < context.units_hold_max and availableCash > bar_dict[context.s].last*context.unit):
                context.tradingSignal = 'entry_add'
            else:
                if (bar_dict[context.s].last > maxx and context.units_hold == 0):
                    context.max_add = bar_dict[context.s].last
                    context.tradingSignal = 'entry'


    atr = getAtrAndUnit(atrArray, atrArray.size, portfolioValue)[0]
    if context.tradedayNum % 5 == 0:
        context.unit = getAtrAndUnit(atrArray, atrArray.size, portfolioValue)[1]
    context.tradedayNum += 1
    context.quantity = context.unit



    if (context.tradingSignal != context.preTradingSignal or (context.units_hold < context.units_hold_max and context.units_hold > 1) or context.tradingSignal == 'stop'):

        if context.tradingSignal == 'entry':
            context.quantity = context.unit
            if availableCash > bar_dict[context.s].last*context.quantity:
                order_shares(context.s, context.quantity)
                context.firstOpenPrice = bar_dict[context.s].last
                context.units_hold = 1


        if context.tradingSignal == 'entry_add':
            context.quantity = context.unit
            order_shares(context.s, context.quantity)
            context.units_hold += 1


        if context.tradingSignal == 'stop':
            if (context.units_hold > 0):
                order_shares(context.s, -context.quantity)
                context.units_hold -= 1


        if context.tradingSignal == 'exit':
            if curPosition > 0:
                order_shares(context.s, -curPosition)
                context.units_hold = 0


    context.preTradingSignal = context.tradingSignal



# -*- coding: utf-8 -*-

from functools import wraps

from contextlib import contextmanager

from ..i18n import gettext as _


class ContextStack(object):

    def __init__(self):
        self.stack = []

    def push(self, obj):
        self.stack.append(obj)

    def pop(self):
        try:
            return self.stack.pop()
        except IndexError:
            raise RuntimeError("stack is empty")

    @contextmanager
    def pushed(self, obj):
        self.push(obj)
        try:
            yield self
        finally:
            self.pop()

    @property
    def top(self):
        try:
            return self.stack[-1]
        except IndexError:
            raise RuntimeError("stack is empty")


class ExecutionContext(object):
    stack = ContextStack()

    def __init__(self, strategy_executor, phase, bar_dict=None):
        """init

        :param StrategyExecutor strategy_executor:
        :param EXECUTION_PHASE phase: current execution phase
        :param BarMap bar_dict: current bar dict
        """
        self.strategy_executor = strategy_executor
        self.phase = phase
        self.bar_dict = bar_dict

    def _push(self):
        self.stack.push(self)

    def _pop(self):
        popped = self.stack.pop()
        if popped is not self:
            raise RuntimeError("Popped wrong context")
        return self

    def __enter__(self):
        self._push()
        return self

    def __exit__(self, _type, _value, _tb):
        """
        Restore the algo instance stored in __enter__.
        """
        self._pop()

    @classmethod
    def get_active(cls):
        return cls.stack.top

    @classmethod
    def get_strategy_context(cls):
        ctx = cls.get_active()
        return ctx.strategy_executor.strategy_context

    @classmethod
    def get_strategy_executor(cls):
        ctx = cls.get_active()
        return ctx.strategy_executor

    @classmethod
    def get_exchange(cls):
        return cls.get_strategy_executor().exchange

    @classmethod
    def get_current_dt(cls):
        return cls.get_strategy_executor().current_dt

    @classmethod
    def get_current_bar_dict(cls):
        ctx = cls.get_active()
        return ctx.bar_dict

    @classmethod
    def get_trading_params(cls):
        return cls.get_exchange().trading_params

    @classmethod
    def enforce_phase(cls, *phases):
        def decorator(func):
            @wraps(func)
            def wrapper(*args, **kwargs):
                if cls.get_active().phase not in phases:
                    raise RuntimeError(
                        _("You can only call %s when executing %s") % (
                            func.__name__,
                            ", ".join(map(lambda x: x.name.lower(), phases))
                        )
                    )
                return func(*args, **kwargs)
            return wrapper
        return decorator



# -*- coding: utf-8 -*-

import os.path
from gettext import NullTranslations, translation


translation_dir = os.path.join(
    os.path.dirname(
        os.path.abspath(
            __file__,
        ),
    ),
    "translations"
)
current_translation = NullTranslations()


def set_locale(locales):
    global current_translation
    current_translation = translation(
        domain="rqalpha",
        localedir=translation_dir,
        languages=locales,
    )


def gettext(message):
    return current_translation.gettext(message)



# -*- coding: utf-8 -*-


class Instrument(object):
    def __init__(self, dic):
        self.__dict__ = dic

    def __repr__(self):
        return "{}({})".format(type(self).__name__,
                               ", ".join(["{}={}".format(k, repr(v))
                                          for k, v in self.__dict__.items()]))



# -*- coding: utf-8 -*-

from collections import defaultdict

from .position import Positions

# TODO make field readonly
# TODO use nametuple to reduce memory


class Dividend(object):
    def __init__(self, order_book_id, quantity, dividend_series):
        self.order_book_id = order_book_id
        self.quantity = quantity
        self.dividend_series = dividend_series


class Portfolio(object):

    def __init__(self):
        self.starting_cash = 0.         # float	回测或实盘交易给算法策略设置的初始资金
        self.cash = 0.                  # float	现在投资组合中剩余的现金
        self.total_returns = 0.         # float	算法投资组合至今的累积百分比收益率。计算方法是现在的投资组合价值/投资组合的初始资金。投资组合价值包含剩余现金和其市场价值。
        self.daily_returns = 0.         # float	当前最新一天的每日收益。
        self.market_value = 0.          # float	投资组合当前的市场价值（未实现/平仓的价值）
        self.portfolio_value = 0.       # float	当前投资组合的总共价值，包含市场价值和剩余现金。
        self.pnl = 0.                   # float	当前投资组合的￥盈亏
        self.annualized_returns = 0.    # float	投资组合的年化收益率
        self.dividend_receivable = 0.   # float	投资组合在分红现金收到账面之前的应收分红部分。具体细节在分红部分
        self.positions = Positions()    # Dictionary	一个包含所有仓位的字典，以id_or_symbol作为键，position对象作为值，关于position的更多的信息可以在下面的部分找到。
        self.start_date = None          # DateTime	策略投资组合的回测/实时模拟交易的开始日期

        self.total_commission = 0.      # float 总的交易费
        self.total_tax = 0.             # float 总的交易税

        self._dividend_info = {}

    def __repr__(self):
        return "Portfolio({0})".format({
            k: v
            for k, v in self.__dict__.items()
            if not k.startswith("_")
        })



# -*- coding: utf-8 -*-
import abc

from six import with_metaclass, iteritems
import pandas as pd

from .bar import BarObject
from ..utils.context import ExecutionContext
from .data_source import LocalDataSource
from ..const import EXECUTION_PHASE


class DataProxy(with_metaclass(abc.ABCMeta)):
    @abc.abstractmethod
    def get_bar(self, order_book_id, dt):
        """get stock Bar object

        :param str order_book_id:
        :param datetime.datetime dt:
        :returns: bar object
        :rtype: BarObject

        """
        raise NotImplementedError

    def latest_bar(self, order_book_id):
        """get latest bar of the stock

        :param str order_book_id:
        :returns: bar object
        :rtype: BarObject

        """
        dt = ExecutionContext.get_current_dt()

        return self.get_bar(order_book_id, dt)

    @abc.abstractmethod
    def get_yield_curve(self, start_date=None, end_date=None):
        """get yield curve of treasure

        :param datetime.date start_date:
        :param datetime.date end_date:
        :returns:
        :rtype: pd.DataFrame

        """
        raise NotImplementedError

    @abc.abstractmethod
    def get_dividends_by_book_date(self, order_book_id, date):
        """get dividend of info by ex date

        :param str order_book_id:
        :param datetime.datetime date:
        :returns: dividend
        :rtype: pd.Series

        """
        raise NotImplementedError

    @abc.abstractmethod
    def history(self, order_book_id, bar_count, frequency, field):
        """get history data

        :param str order_book_id:
        :param int bar_count:
        :param str frequency: '1d' or '1m'
        :param str field: "open", "close", "high", "low", "volume", "last", "total_turnover"
        :returns:
        :rtype: pandas.DataFrame

        """
        raise NotImplementedError

    @abc.abstractmethod
    def instrument(self, order_book_id):
        """get instrument of order book id

        :param str order_book_id:
        :returns: result instrument
        :rtype: Instrument

        """
        raise NotImplementedError


class LocalDataProxy(DataProxy):
    def __init__(self, root_dir):
        self._data_source = LocalDataSource(root_dir)
        self._cache = {}
        self._dividend_cache = {}

    def get_bar(self, order_book_id, dt):
        try:
            df = self._cache[order_book_id]
        except KeyError:
            df = self._data_source.get_all_bars(order_book_id)
            self._cache[order_book_id] = df

        return BarObject(self._data_source.instruments(order_book_id), df.xs(dt.date()))

    def history(self, order_book_id, bar_count, frequency, field):
        if frequency == '1m':
            raise RuntimeError('Minute bar not supported yet!')

        try:
            df = self._cache[order_book_id]
        except KeyError:
            df = self._data_source.get_all_bars(order_book_id)
            self._cache[order_book_id] = df

        dt = ExecutionContext.get_current_dt().date()
        i = df.index.searchsorted(dt, side='right')
        # you can only access yesterday history in before_trading
        if ExecutionContext.get_active().phase == EXECUTION_PHASE.BEFORE_TRADING:
            i -= 1
        left = i - bar_count if i >= bar_count else 0
        return df[left:i][field]

    def get_yield_curve(self, start_date=None, end_date=None):
        return self._data_source.get_yield_curve(start_date, end_date)

    def get_dividends_by_book_date(self, order_book_id, date):
        if order_book_id not in self._dividend_cache:
            dividend_df = self._data_source.get_dividends(order_book_id)
            if not dividend_df.empty:
                dividend_df.set_index("book_closure_date", inplace=True)
            self._dividend_cache[order_book_id] = dividend_df

        dividend_df = self._dividend_cache[order_book_id]
        try:
            series = dividend_df.ix[date]
            return series
        except KeyError:
            return None

    def get_trading_dates(self, start_date, end_date):
        return self._data_source.get_trading_dates(start_date, end_date)

    def instrument(self, order_book_id):
        return self._data_source.instruments(order_book_id)



# -*- coding: utf-8 -*-

import datetime
import os

import pandas as pd
import pytest
import pytz
from six import print_

from rqalpha.data import LocalDataProxy
from rqalpha.trading_params import TradingParams
from rqalpha.analyser.simulation_exchange import SimuExchange


@pytest.fixture()
def data_proxy():
    return LocalDataProxy(os.environ.get("RQ_LOCAL_STORE", os.path.expanduser("~/.rqalpha")))


@pytest.fixture()
def trading_calendar():
    trading_cal = data_proxy().get_trading_dates("2005-01-01", "2020-01-01")

    return trading_cal


@pytest.fixture()
def trading_params():
    trading_cal = trading_calendar()

    trading_cal = trading_cal[
        (trading_cal >= "2013-02-01") & (trading_cal <= "2013-05-01")
    ]

    params = TradingParams(trading_cal)
    return params



# -*- coding: utf-8 -*-

import copy

import click
import six
from six import iteritems
import pandas as pd

from .analyser.simulation_exchange import SimuExchange
from .const import EVENT_TYPE, EXECUTION_PHASE
from .data import BarMap
from .events import SimulatorAStockTradingEventSource
from .utils import ExecutionContext, dummy_func
from .scheduler import scheduler
from .analyser.commission import AStockCommission
from .analyser.slippage import FixedPercentSlippageDecider


class StrategyContext(object):
    def __init__(self):
        self.__last_portfolio_update_dt = None

    @property
    def now(self):
        return ExecutionContext.get_current_dt()

    @property
    def slippage(self):
        return copy.deepcopy(ExecutionContext.get_exchange().account.slippage_decider)

    @slippage.setter
    @ExecutionContext.enforce_phase(EXECUTION_PHASE.INIT)
    def slippage(self, value):
        assert isinstance(value, (int, float))

        ExecutionContext.get_exchange().account.slippage_decider = FixedPercentSlippageDecider(rate=value)

    @property
    def commission(self):
        return copy.deepcopy(ExecutionContext.get_exchange().account.commission_decider)

    @commission.setter
    @ExecutionContext.enforce_phase(EXECUTION_PHASE.INIT)
    def commission(self, value):
        assert isinstance(value, (int, float))

        ExecutionContext.get_exchange().account.commission_decider = AStockCommission(commission_rate=value)

    @property
    def benchmark(self):
        return copy.deepcopy(ExecutionContext.get_trading_params().benchmark)

    @benchmark.setter
    @ExecutionContext.enforce_phase(EXECUTION_PHASE.INIT)
    def benchmark(self, value):
        assert isinstance(value, six.string_types)

        ExecutionContext.get_trading_params().benchmark = value

    @property
    def short_selling_allowed(self):
        raise NotImplementedError

    @short_selling_allowed.setter
    def short_selling_allowed(self):
        raise NotImplementedError

    @property
    def portfolio(self):
        dt = self.now
        # if self.__last_portfolio_update_dt != dt:
        # FIXME need to use cache, or might use proxy rather then copy
        if True:
            self.__portfolio = copy.deepcopy(ExecutionContext.get_exchange().account.portfolio)
            self.__last_portfolio_update_dt = dt
        return self.__portfolio

    def __repr__(self):
        items = ("%s = %r" % (k, v)
                 for k, v in self.__dict__.items()
                 if not callable(v) and not k.startswith("_"))
        return "Context({%s})" % (', '.join(items), )


class StrategyExecutor(object):
    def __init__(self, trading_params, data_proxy, **kwargs):
        """init

        :param Strategy strategy: current user strategy object
        :param TradingParams trading_params: current trading params
        :param DataProxy data_proxy: current data proxy to access data
        """
        self.trading_params = trading_params
        self._data_proxy = data_proxy

        self._strategy_context = kwargs.get("strategy_context")
        if self._strategy_context is None:
            self._strategy_context = StrategyContext()

        self._user_init = kwargs.get("init", dummy_func)
        self._user_handle_bar = kwargs.get("handle_bar", dummy_func)
        self._user_before_trading = kwargs.get("before_trading", dummy_func)

        self._simu_exchange = kwargs.get("simu_exchange")
        if self._simu_exchange is None:
            self._simu_exchange = SimuExchange(data_proxy, trading_params)

        self._event_source = SimulatorAStockTradingEventSource(trading_params)
        self._current_dt = None
        self.current_universe = set()

        self.progressbar = click.progressbar(length=len(self.trading_params.trading_calendar), show_eta=False)

    def execute(self):
        """run strategy

        :returns: performance results
        :rtype: pandas.DataFrame

        """
        # use local variable for performance
        data_proxy = self.data_proxy
        strategy_context = self.strategy_context
        simu_exchange = self.exchange

        init = self._user_init
        before_trading = self._user_before_trading
        handle_bar = self._user_handle_bar

        exchange_on_dt_change = simu_exchange.on_dt_change
        exchange_on_bar_close = simu_exchange.on_bar_close
        exchange_on_day_open = simu_exchange.on_day_open
        exchange_on_day_close = simu_exchange.on_day_close
        exchange_update_portfolio = simu_exchange.update_portfolio

        is_show_progress_bar = self.trading_params.show_progress

        def on_dt_change(dt):
            self._current_dt = dt
            exchange_on_dt_change(dt)

        with ExecutionContext(self, EXECUTION_PHASE.INIT):
            init(strategy_context)

        for dt, event in self._event_source:
            on_dt_change(dt)

            bar_dict = BarMap(dt, self.current_universe, data_proxy)

            if event == EVENT_TYPE.DAY_START:
                with ExecutionContext(self, EXECUTION_PHASE.BEFORE_TRADING, bar_dict):
                    exchange_on_day_open()
                    before_trading(strategy_context, None)

            elif event == EVENT_TYPE.HANDLE_BAR:
                with ExecutionContext(self, EXECUTION_PHASE.HANDLE_BAR, bar_dict):
                    exchange_update_portfolio(bar_dict)
                    handle_bar(strategy_context, bar_dict)
                    scheduler.next_day(dt, strategy_context, bar_dict)
                    exchange_on_bar_close(bar_dict)

            elif event == EVENT_TYPE.DAY_END:
                with ExecutionContext(self, EXECUTION_PHASE.FINALIZED, bar_dict):
                    exchange_on_day_close()

                if is_show_progress_bar:
                    self.progressbar.update(1)

        results_df = self.generate_result(simu_exchange)

        return results_df

    def generate_result(self, simu_exchange):
        """generate result dataframe

        :param simu_exchange:
        :returns: result dataframe contains daliy portfolio, risk and trades
        :rtype: pd.DataFrame
        """
        account = simu_exchange.account
        risk_cal = simu_exchange.risk_cal
        columns = [
            "daily_returns",
            "total_returns",
            "annualized_returns",
            "market_value",
            "portfolio_value",
            "total_commission",
            "total_tax",
            "pnl",
            "positions",
            "cash",
        ]
        risk_keys = [
            "volatility", "max_drawdown",
            "alpha", "beta", "sharpe",
            "information_rate", "downside_risk",
            "tracking_error", "sortino",
        ]

        data = []
        for date, portfolio in iteritems(simu_exchange.daily_portfolios):
            # portfolio
            items = {"date": pd.Timestamp(date)}
            for key in columns:
                items[key] = getattr(portfolio, key)

            # trades
            items["trades"] = account.get_all_trades()[date]

            # risk
            risk = risk_cal.daily_risks[date]
            for risk_key in risk_keys:
                items[risk_key] = getattr(risk, risk_key)

            idx = risk_cal.trading_index.get_loc(date)
            items["benchmark_total_returns"] = risk_cal.benchmark_total_returns[idx]
            items["benchmark_daily_returns"] = risk_cal.benchmark_total_daily_returns[idx]
            items["benchmark_annualized_returns"] = risk_cal.benchmark_annualized_returns[idx]

            data.append(items)

        results_df = pd.DataFrame(data)
        results_df.set_index("date", inplace=True)

        return results_df

    @property
    def strategy_context(self):
        """get current strategy

        :returns: current strategy
        :rtype: Strategy
        """
        return self._strategy_context

    @property
    def exchange(self):
        """get current exchange

        :returns: current exchange
        :rtype: SimuExchange
        """
        return self._simu_exchange

    @property
    def data_proxy(self):
        """get current data proxy

        :returns: current data proxy
        :rtype: DataProxy
        """
        return self._data_proxy

    @property
    def current_dt(self):
        """get current simu datetime

        :returns: current datetime
        :rtype: datetime.datetime
        """
        return self._current_dt



# -*- coding: utf-8 -*-
import datetime


class Scheduler:
    TRADING_DATES = None

    @classmethod
    def set_trading_dates(cls, trading_dates):
        cls.TRADING_DATES = trading_dates

    def __init__(self):
        self._registry = []
        self._today = None
        self._this_week = None
        self._this_month = None

    def _always_true(self):
        return True

    def _is_weekday(self, wd):
        return self._today.weekday() == wd

    def _is_nth_trading_day_in_week(self, n):
        try:
            return self._this_week[n] == self._today
        except IndexError:
            return False

    def _is_nth_trading_day_in_month(self, n):
        try:
            return self._this_month[n] == self._today
        except IndexError:
            return False

    def run_daily(self, func):
        self._registry.append((self._always_true, func))

    def run_weekly(self, func, weekday=None, tradingday=None):
        if (weekday is not None and tradingday is not None) or (weekday is None and tradingday is None):
            raise ValueError('select one of weekday/tradingday')

        if weekday is not None:
            if weekday < 1 or weekday > 7:
                raise ValueError('invalid weekday, should be in [1, 7]')
            self._registry.append((lambda: self._is_weekday(weekday - 1), func))
        else:
            if tradingday > 5 or tradingday < -5 or tradingday == 0:
                raise ValueError('invalid trading day, should be in [-5, 0), (0, 5]')
            if tradingday > 0:
                tradingday -= 1
            self._registry.append((lambda: self._is_nth_trading_day_in_week(tradingday), func))

    def run_monthly(self, func, tradingday):
        if tradingday > 23 or tradingday < -23 or tradingday == 0:
            raise ValueError('invalid tradingday, should be in [-23, 0), (0, 23]')
        if tradingday > 0:
            tradingday -= 1
        self._registry.append((lambda: self._is_nth_trading_day_in_month(tradingday), func))

    def next_day(self, dt, context, bars):
        if len(self._registry) == 0:
            return

        self._today = dt.date()
        if not self._this_week or self._today > self._this_week[-1]:
            self._fill_week()
        if not self._this_month or self._today > self._this_month[-1]:
            self._fill_month()

        for rule, func in self._registry:
            if rule():
                func(context, bars)

    def _fill_week(self):
        weekday = self._today.isoweekday()
        weekend = self._today + datetime.timedelta(days=7-weekday)
        week_start = weekend - datetime.timedelta(days=6)

        left = self.TRADING_DATES.searchsorted(week_start)
        right = self.TRADING_DATES.searchsorted(weekend, side='right')
        self._this_week = [d.date() for d in self.TRADING_DATES[left:right]]

    def _fill_month(self):
        try:
            month_end = self._today.replace(month=self._today.month+1, day=1)
        except ValueError:
            month_end = self._today.replace(year=self._today.year+1, month=1, day=1)

        left, right = self.TRADING_DATES.searchsorted(self._today), self.TRADING_DATES.searchsorted(month_end)
        self._this_month = [d.date() for d in self.TRADING_DATES[left:right]]


scheduler = Scheduler()



# -*- coding: utf-8 -*-

from rqalpha.events import SimulatorAStockTradingEventSource
from rqalpha.const import EVENT_TYPE
from rqalpha.trading_params import TradingParams

from .fixture import *


def test_event_source(trading_params):
    source = SimulatorAStockTradingEventSource(trading_params)

    assert len(trading_params.trading_calendar) * len(EVENT_TYPE) == len(list(source))



# -*- coding: utf-8 -*-

import datetime

import click
import pandas as pd


class Date(click.ParamType):
    def __init__(self, tz=None):
        self.tz = tz

    def convert(self, value, param, ctx):
        return pd.Timestamp(value, self.tz)

    @property
    def name(self):
        return type(self).__name__.upper()



# -*- coding: utf-8 -*-
import datetime

from rqalpha.strategy import StrategyExecutor
from rqalpha.api import *
from .fixture import *


def test_schedule(trading_params, data_proxy):
    run_at = []

    def should_not_call(c, d):
        raise RuntimeError('should not call')

    def is_weekday(dt, weekday):
        assert(dt.isoweekday() == weekday)

    def week_of(dt):
        weekday = dt.isoweekday()
        weekend = dt + datetime.timedelta(days=7-weekday)
        start = weekend - datetime.timedelta(days=6)
        return start, weekend

    def is_nth_trading_day_in_week(dt, nth):
        week = data_proxy.get_trading_dates(*week_of(dt))
        if nth > 0:
            nth -= 1
        assert(dt == week[nth].date())

    def month_of(dt):
        start = dt.replace(day=1)
        try:
            end = dt.replace(month=dt.month+1, day=1)
        except ValueError:
            end = dt.replace(year=dt.year+1, month=1, day=1)
        end = end - datetime.timedelta(1)
        return start, end

    def is_nth_trading_day_in_month(dt, nth):
        month = data_proxy.get_trading_dates(*month_of(dt))
        if nth > 0:
            nth -= 1
        assert(dt == month[nth].date())

    def init(context):
        scheduler.run_daily(lambda c, bar_dict: run_at.append(c.now))
        for i in range(1, 8):
            scheduler.run_weekly((lambda n: lambda c, _: is_weekday(c.now.date(), n))(i), weekday=i)

        for i in range(-5, 0):
            scheduler.run_weekly((lambda n: lambda c, _: is_nth_trading_day_in_week(c.now.date(), n))(i), tradingday=i)

        for i in range(1, 6):
            scheduler.run_weekly((lambda n: lambda c, _: is_nth_trading_day_in_week(c.now.date(), n))(i), tradingday=i)

        for i in range(-23, 0):
            scheduler.run_monthly((lambda n: lambda c, _: is_nth_trading_day_in_month(c.now.date(), n))(i),
                                  tradingday=i)

        for i in range(1, 24):
            scheduler.run_monthly((lambda n: lambda c, _: is_nth_trading_day_in_month(c.now.date(), n))(i),
                                  tradingday=i)

    def before_trading(context, bar_dict):
        pass

    def handle_bar(context, bar_dict):
        pass

    executor = StrategyExecutor(
        init=init,
        before_trading=before_trading,
        handle_bar=handle_bar,

        trading_params=trading_params,
        data_proxy=data_proxy,
    )

    scheduler.set_trading_dates(data_proxy.get_trading_dates('2005-01-04', datetime.date.today()))

    perf = executor.execute()
    assert(len(run_at) == len(trading_params.trading_calendar))



# -*- coding: utf-8 -*-

from pprint import pprint

import numpy as np
from six import iteritems

from rqalpha import StrategyExecutor
from rqalpha.api import order_shares
from rqalpha.trading_params import TradingParams
from .fixture import *


def test_buy_and_sell(trading_calendar, data_proxy):
    def init(context):
        context.cnt = 0

    def handle_bar(context, bar_dict):
        context.cnt += 1

        if context.cnt % 2 == 1:
            order_shares("000001.XSHE", 5000)
        else:
            order_shares("000001.XSHE", -5000)


    trading_cal = trading_calendar
    trading_cal = trading_cal[
        (trading_cal >= "2013-02-01") & (trading_cal <= "2013-05-01")
    ]
    trading_params = TradingParams(trading_cal)

    executor = StrategyExecutor(
        init=init,
        handle_bar=handle_bar,

        trading_params=trading_params,
        data_proxy=data_proxy,
    )

    perf = executor.execute()

    portfolio = executor.exchange.account.portfolio
    assert np.isclose(portfolio.annualized_returns, -0.094760148802936484)
    assert np.isclose(portfolio.total_returns, -0.022917433828999911)


def test_dividend(trading_calendar, data_proxy):
    def init(context):
        context.cnt = 0

    def handle_bar(context, bar_dict):
        context.cnt += 1

        if context.cnt < 3:
            order_shares("000001.XSHE", 5000)

    trading_cal = trading_calendar
    trading_cal = trading_cal[
        (trading_cal >= "2013-02-01") & (trading_cal <= "2013-07-01")
    ]
    trading_params = TradingParams(trading_cal)

    executor = StrategyExecutor(
        init=init,
        handle_bar=handle_bar,

        trading_params=trading_params,
        data_proxy=data_proxy,
    )

    perf = executor.execute()

    portfolio = executor.exchange.account.portfolio

    assert np.isclose(portfolio.annualized_returns, -0.47051001393709346)
    assert np.isclose(portfolio.total_returns, -0.23129389564400005)



# -*- coding: utf-8 -*-
from __future__ import division
import copy
from collections import OrderedDict

import pandas as pd
import numpy as np

from .risk import Risk
from .. import const


class RiskCal(object):
    def __init__(self, trading_params, data_proxy):
        self.data_proxy = data_proxy

        self.start_date = trading_params.start_date
        self.trading_index = trading_params.trading_calendar
        self.trading_days_cnt = len(self.trading_index)

        self.strategy_total_daily_returns = np.full(self.trading_days_cnt, np.nan)
        self.benchmark_total_daily_returns = np.full(self.trading_days_cnt, np.nan)
        self.strategy_current_daily_returns = None
        self.benchmark_current_daily_returns = None

        self.strategy_total_returns = np.full(self.trading_days_cnt, np.nan)
        self.benchmark_total_returns = np.full(self.trading_days_cnt, np.nan)
        self.strategy_current_total_returns = None
        self.benchmark_current_total_returns = None

        self.strategy_annualized_returns = np.full(self.trading_days_cnt, np.nan)
        self.benchmark_annualized_returns = np.full(self.trading_days_cnt, np.nan)
        self.strategy_current_annualized_returns = None
        self.benchmark_current_annualized_returns = None

        self.risk = Risk()

        self.daily_risks = OrderedDict()

        self.current_max_returns = -np.inf
        self.current_max_drawdown = 0

        # FIXME might change daily?
        self.risk_free_rate = data_proxy.get_yield_curve(self.trading_index[0], self.trading_index[-1])

    def calculate(self, date, strategy_daily_returns, benchmark_daily_returns):
        idx = self.latest_idx = self.trading_index.get_loc(date)

        # daily
        self.strategy_total_daily_returns[idx] = strategy_daily_returns
        self.benchmark_total_daily_returns[idx] = benchmark_daily_returns
        self.strategy_current_daily_returns = self.strategy_total_daily_returns[:idx + 1]
        self.benchmark_current_daily_returns = self.benchmark_total_daily_returns[:idx + 1]

        self.days_cnt = len(self.strategy_current_daily_returns)
        days_pass_cnt = (date - self.start_date).days + 1

        # risk
        # self.riskfree_total_returns = self.risk_free_rate / self.days_cnt * const.DAYS_CNT.TRADING_DAYS_A_YEAR
        self.riskfree_total_returns = self.risk_free_rate

        # total
        self.strategy_total_returns[idx] = (1. + self.strategy_current_daily_returns).prod() - 1
        self.benchmark_total_returns[idx] = (1. + self.benchmark_current_daily_returns).prod() - 1
        self.strategy_current_total_returns = self.strategy_total_returns[:idx + 1]
        self.benchmark_current_total_returns = self.benchmark_total_returns[:idx + 1]

        # annual
        self.strategy_annualized_returns[idx] = (1 + self.strategy_current_total_returns[-1]) ** (
                    const.DAYS_CNT.DAYS_A_YEAR / days_pass_cnt) - 1
        self.benchmark_annualized_returns[idx] = (1 + self.benchmark_current_total_returns[-1]) ** (
            const.DAYS_CNT.DAYS_A_YEAR / days_pass_cnt) - 1
        self.strategy_current_annualized_returns = self.strategy_annualized_returns[:idx + 1]
        self.benchmark_current_annualized_returns = self.benchmark_annualized_returns[:idx + 1]

        if self.strategy_current_total_returns[-1] > self.current_max_returns:
            self.current_max_returns = self.strategy_current_total_returns[-1]

        risk = self.risk
        risk.volatility = self.cal_volatility()
        risk.max_drawdown = self.cal_max_drawdown()
        risk.tracking_error = self.cal_tracking_error()
        risk.information_rate = self.cal_information_rate(risk.volatility)
        risk.downside_risk = self.cal_downside_risk()
        risk.beta = self.cal_beta()
        risk.alpha = self.cal_alpha()
        risk.sharpe = self.cal_sharpe()
        risk.sortino = self.cal_sortino()

        self.daily_risks[date] = copy.deepcopy(risk)

    def cal_volatility(self):
        daily_returns = self.strategy_current_daily_returns
        if len(daily_returns) <= 1:
            return 0.
        volatility = const.DAYS_CNT.TRADING_DAYS_A_YEAR ** 0.5 * np.std(daily_returns, ddof=1)
        return volatility

    def cal_max_drawdown(self):
        today_return = self.strategy_current_total_returns[-1]
        today_drawdown = (1. + today_return) / (1. + self.current_max_returns) - 1.
        if today_drawdown < self.current_max_drawdown:
            self.current_max_drawdown = today_drawdown
        return self.current_max_drawdown

    def cal_tracking_error(self):
        diff = self.strategy_current_daily_returns - self.benchmark_current_daily_returns
        return ((diff * diff).sum() / len(diff)) ** 0.5 * const.DAYS_CNT.TRADING_DAYS_A_YEAR ** 0.5

    def cal_information_rate(self, volatility):
        strategy_rets = self.strategy_current_daily_returns.sum() / len(self.strategy_current_daily_returns) * const.DAYS_CNT.TRADING_DAYS_A_YEAR
        benchmark_rets = self.benchmark_current_daily_returns.sum() / len(self.benchmark_current_daily_returns) * const.DAYS_CNT.TRADING_DAYS_A_YEAR

        return (strategy_rets - benchmark_rets) / volatility

    def cal_alpha(self):
        beta = self.risk.beta

        strategy_rets = self.strategy_current_daily_returns.sum() / len(self.strategy_current_daily_returns) * const.DAYS_CNT.TRADING_DAYS_A_YEAR
        benchmark_rets = self.benchmark_current_daily_returns.sum() / len(self.benchmark_current_daily_returns) * const.DAYS_CNT.TRADING_DAYS_A_YEAR

        alpha = strategy_rets - (self.riskfree_total_returns + beta * (benchmark_rets - self.riskfree_total_returns))
        return alpha

    def cal_beta(self):
        if len(self.strategy_current_daily_returns) <= 1:
            return 0.
        cov = np.cov(np.vstack([
            self.strategy_current_daily_returns,
            self.benchmark_current_daily_returns,
        ]), ddof=1)
        beta = cov[0][1] / cov[1][1]

        return beta

    def cal_sharpe(self):
        volatility = self.risk.volatility
        strategy_rets = self.strategy_current_daily_returns.sum() / len(self.strategy_current_daily_returns) * const.DAYS_CNT.TRADING_DAYS_A_YEAR

        sharpe = (strategy_rets - self.riskfree_total_returns) / volatility

        return sharpe

    def cal_sortino(self):
        strategy_rets = self.strategy_current_daily_returns.sum() / len(self.strategy_current_daily_returns) * const.DAYS_CNT.TRADING_DAYS_A_YEAR
        downside_risk = self.risk.downside_risk

        sortino = (strategy_rets - self.riskfree_total_returns) / downside_risk
        return sortino

    def cal_downside_risk(self):
        mask = self.strategy_current_daily_returns < self.benchmark_current_daily_returns
        diff = self.strategy_current_daily_returns[mask] - self.benchmark_current_daily_returns[mask]
        if len(diff) <= 1:
            return 0.

        return ((diff * diff).sum() / len(diff)) ** 0.5 * const.DAYS_CNT.TRADING_DAYS_A_YEAR ** 0.5

    def __repr__(self):
        return "RiskCal({0})".format(self.__dict__)



# -*- coding: utf-8 -*-

import pytz
import pandas as pd


class TradingParams(object):
    def __init__(self, trading_calendar, **kwargs):
        assert isinstance(trading_calendar, pd.Index)
        self.trading_calendar = trading_calendar
        self.timezone = kwargs.get("timezone", pytz.utc)
        self.benchmark = kwargs.get("benchmark", "000300.XSHG")
        self.frequency = kwargs.get("frequency", "1d")

        self.start_date = kwargs.get("start_date", self.trading_calendar[0].date())
        self.end_date = kwargs.get("end_date", self.trading_calendar[-1].date())

        self.init_cash = kwargs.get("init_cash", 100000)

        self.show_progress = kwargs.get("show_progress", False)



# -*- coding: utf-8 -*-
import abc

from six import with_metaclass, iteritems


class BaseCommission(with_metaclass(abc.ABCMeta)):
    @abc.abstractmethod
    def get_commission(self, data_proxy, order):
        raise NotImplementedError


class AStockCommission(BaseCommission):
    def __init__(self, commission_rate=0.0008, min_commission=5):
        self.commission_rate = commission_rate
        self.min_commission = min_commission

    def get_commission(self, order, trade):
        cost_money = trade.price * abs(trade.amount)
        v = cost_money * self.commission_rate
        v = max(self.min_commission, v)

        return v



# -*- coding: utf-8 -*-
import abc

from six import with_metaclass


class OrderStyle(with_metaclass(abc.ABCMeta)):
    @abc.abstractmethod
    def get_limit_price(self, is_buy):
        raise NotImplementedError


class MarketOrder(OrderStyle):

    def get_limit_price(self, _is_buy):
        return None


class LimitOrder(OrderStyle):

    def __init__(self, limit_price):
        self.limit_price = limit_price

    def get_limit_price(self, is_buy):
        return self.limit_price



# -*- coding: utf-8 -*-

import copy
from collections import defaultdict, OrderedDict

import pandas as pd
import numpy as np
from six import iteritems

from ..const import ORDER_STATUS
from .. import const
from ..account import Account
from ..data import BarMap
from ..i18n import gettext as _
from ..logger import user_log
from ..utils.context import ExecutionContext
from .commission import AStockCommission
from .slippage import FixedPercentSlippageDecider
from .tax import AStockTax
from .order import Order
from .order_style import MarketOrder, LimitOrder
from .portfolio import Portfolio, Dividend
from .position import Position
from .risk_cal import RiskCal
from .trade import Trade


class SimuExchange(object):
    def __init__(self, data_proxy, trading_params, **kwargs):
        self.data_proxy = data_proxy
        self.trading_params = trading_params

        self.dt = None          # type: datetime.datetime current simulation datetime

        # TODO move risk cal outside this class
        self.risk_cal = RiskCal(trading_params, data_proxy)

        self.daily_portfolios = OrderedDict()      # type: Dict[str, Portfolio], each day has a portfolio copy
        self.all_orders = {}                       # type: Dict[str, Order], all orders, including cancel orders
        self.open_orders = defaultdict(list)       # type: Dict[str, List[Order]], all open orders

        self.start_date = start_date = self.trading_params.trading_calendar[0].date()
        self.account = Account(start_date=start_date, init_cash=self.trading_params.init_cash)

        # TODO should treat benchmark as a buy and hold strategy
        self.benchmark_portfolio_value = self.benchmark_cash = self.trading_params.init_cash
        self.benchmark_market_value = 0
        self.benchmark_quantity = 0

        self.last_date = None        # type: datetime.date, last trading date
        self.simu_days_cnt = 0       # type: int, days count since simulation start

    def on_dt_change(self, dt):
        if dt.date() != self.current_date:
            self.last_date = self.current_date

        self.dt = dt

    @property
    def current_date(self):
        return self.dt.date() if self.dt else None

    def on_day_open(self):
        self.handle_dividend_payable()

    def on_day_close(self):
        self.simu_days_cnt += 1

        self.reject_all_open_orders()

        trades = self.account.get_trades(self.current_date)
        portfolio = self.account.portfolio
        positions = portfolio.positions

        # update position sellable for T+1
        for trade in trades:
            position = positions[trade.order_book_id]
            position.sellable += trade.amount

        self.update_daily_portfolio()

        for order_book_id in list(positions.keys()):
            position = positions[order_book_id]
            if position.quantity == 0:
                positions.pop(order_book_id)

        # store today portfolio
        self.daily_portfolios[self.current_date] = copy.deepcopy(portfolio)

        # TODO make benchmark cal works better
        # update benchmark
        if self.benchmark_market_value == 0:
            self.benchmark_market_value = None

            origin_benchmark_portfolio_value = self.benchmark_portfolio_value

            # FIXME quick dirty hack
            price = self.data_proxy.get_bar(self.trading_params.benchmark, pd.Timestamp(self.start_date)).close
            self.benchmark_quantity = self.benchmark_portfolio_value / price
            self.benchmark_quantity = int(self.benchmark_quantity)
            trade_price = price
            commission = 0.0008 * trade_price * self.benchmark_quantity

            self.benchmark_cash -= trade_price * self.benchmark_quantity
            self.benchmark_cash -= commission

            self.benchmark_market_value = price * self.benchmark_quantity
            self.benchmark_portfolio_value = self.benchmark_market_value + self.benchmark_cash

            benchmark_daily_returns = self.benchmark_portfolio_value / origin_benchmark_portfolio_value - 1
        else:
            new_benchmark_market_value = self.data_proxy.get_bar(
                self.trading_params.benchmark, pd.Timestamp(self.current_date)).close * self.benchmark_quantity
            new_benchmark_portfolio_value = new_benchmark_market_value + self.benchmark_cash

            benchmark_daily_returns = new_benchmark_portfolio_value / self.benchmark_portfolio_value - 1
            self.benchmark_portfolio_value = new_benchmark_portfolio_value

        self.risk_cal.calculate(self.current_date, portfolio.daily_returns, benchmark_daily_returns)

        self.handle_dividend_ex_dividend()

    def get_yesterday_portfolio(self):
        return self.daily_portfolios.get(self.last_date)

    def reject_all_open_orders(self):
        for order_book_id, order_list in iteritems(self.open_orders):
            for order in order_list:
                user_log.warn(_("Order Rejected: {order_book_id} can not match, {order_list}").format(
                    order_book_id=order_book_id,
                    order_list=order_list,
                ))
                order.mark_rejected(_("market close"))
            del order_list[:]

    def match_current_orders(self, bar_dict):
        trades, close_orders = self.match_orders(bar_dict)

        for trade in trades:
            self.account.record_new_trade(self.current_date, trade)

        self.remove_close_orders(close_orders)

        # remove rejected order
        rejected_orders = []
        for order_book_id, order_list in iteritems(self.open_orders):
            for order in order_list:
                if order.status == ORDER_STATUS.REJECTED:
                    rejected_orders.append(order)

        self.remove_close_orders(rejected_orders)

    def on_bar_close(self, bar_dict):
        self.match_orders(bar_dict)
        # self.update_portfolio(bar_dict)

    def update_daily_portfolio(self):
        yesterday_portfolio = self.get_yesterday_portfolio()
        portfolio = self.account.portfolio

        if yesterday_portfolio is None:
            yesterday_portfolio_value = portfolio.starting_cash
        else:
            yesterday_portfolio_value = yesterday_portfolio.portfolio_value

        portfolio.pnl = portfolio.portfolio_value - yesterday_portfolio_value
        portfolio.daily_returns = portfolio.pnl / yesterday_portfolio_value
        portfolio.total_returns = portfolio.portfolio_value / portfolio.starting_cash - 1
        portfolio.annualized_returns = (1 + portfolio.total_returns) ** (
            const.DAYS_CNT.DAYS_A_YEAR / float((self.current_date - self.trading_params.start_date).days + 1)) - 1

    def update_portfolio(self, bar_dict):
        portfolio = self.account.portfolio
        positions = portfolio.positions

        for order_book_id, position in iteritems(positions):
            position.market_value = position.quantity * bar_dict[order_book_id].close

        portfolio.market_value = sum(position.market_value for order_book_id, position in iteritems(positions))
        portfolio.portfolio_value = portfolio.market_value + portfolio.cash

        for order_book_id, position in iteritems(positions):
            position.value_percent = position.market_value / portfolio.portfolio_value

    def create_order(self, bar_dict, order_book_id, amount, style):
        if style is None:
            style = MarketOrder()

        order = Order(self.dt, order_book_id, amount, style)

        self.open_orders[order_book_id].append(order)
        self.all_orders[order.order_id] = order

        # match order here because ricequant do this
        self.match_current_orders(bar_dict)
        self.update_portfolio(bar_dict)

        return order

    def cancel_order(self, order_id):
        order = self.get_order(order_id)
        if order in self.open_orders[order.order_book_id]:
            order.cancel()

    def remove_close_orders(self, close_orders):
        for order in close_orders:
            order_list = self.open_orders[order.order_book_id]
            try:
                order_list.remove(order)
            except ValueError:
                pass

    def get_order(self, order_id):
        return self.all_orders[order_id]

    def match_orders(self, bar_dict):
        # TODO abstract Matching Engine
        trades = []
        close_orders = []

        portfolio = self.account.portfolio
        positions = portfolio.positions

        slippage_decider = self.account.slippage_decider
        commission_decider = self.account.commission_decider
        tax_decider = self.account.tax_decider
        data_proxy = self.data_proxy

        for order_book_id, order_list in iteritems(self.open_orders):
            # TODO handle limit order
            for order in order_list:
                # TODO check whether can match
                is_pass, reason = self.validate_order(bar_dict, order)
                if not is_pass:
                    order.mark_rejected(reason)
                    user_log.error(reason)
                    continue

                trade_price = slippage_decider.get_trade_price(data_proxy, order)
                amount = order.quantity

                trade = Trade(
                    date=order.dt,
                    order_book_id=order_book_id,
                    price=trade_price,
                    amount=order.quantity,
                    order_id=order.order_id,
                    commission=0.,
                )

                commission = commission_decider.get_commission(order, trade)
                trade.commission = commission

                tax = tax_decider.get_tax(order, trade)
                trade.tax = tax

                # deduct available cash
                portfolio.cash -= trade_price * amount

                # cal commisssion & tax
                portfolio.cash -= commission
                portfolio.cash -= tax
                portfolio.total_commission += commission
                portfolio.total_tax += tax

                # update order
                # TODO simu to create more trades
                order.filled_shares = order.quantity

                close_orders.append(order)
                trades.append(trade)

                # update position
                position = positions[order_book_id]
                position.quantity += trade.amount

                if trade.amount > 0:
                    position.bought_quantity += trade.amount
                    position.bought_value += trade_price * amount
                else:
                    position.sold_quantity += abs(trade.amount)
                    position.sold_value += abs(trade_price * amount)

        return trades, close_orders

    def validate_order(self, bar_dict, order):
        # TODO need to be abstract as a validator

        order_book_id = order.order_book_id

        portfolio = self.account.portfolio
        positions = portfolio.positions
        position = positions[order_book_id]

        bar = bar_dict[order_book_id]

        amount = order.quantity
        close_price = bar.close
        price = self.account.slippage_decider.get_trade_price(self.data_proxy, order)
        cost_money = price * amount
        is_buy = amount > 0

        # handle limit order
        if self.trading_params.frequency == "1d":
            if isinstance(order.style, LimitOrder):
                limit_price = order.style.get_limit_price(is_buy)
                if is_buy and limit_price < bar.close:
                    return False, _("Order Rejected: price is to low to buy {order_book_id}").format(
                        order_book_id=order_book_id)
                elif not is_buy and limit_price > bar.close:
                    return False, _("Order Rejected: price is to high to sell {order_book_id}").format(
                        order_book_id=order_book_id)
        else:
            raise NotImplementedError

        # check amount
        if abs(amount) < 100:
            return False, _("Order Rejected: amount must over 100 for {order_book_id} ").format(
                order_book_id=order_book_id,
            )

        # check money is enough
        if is_buy and close_price * amount > self.account.portfolio.cash:
            return False, _("Order Rejected: no enough money to buy {order_book_id}, needs {cost_money:.2f}, cash {cash:.2f}").format(
                order_book_id=order_book_id,
                cost_money=cost_money,
                cash=portfolio.cash,
            )

        # TODO check whether is enough to sell
        if order.quantity < 0 and abs(order.quantity) > position.sellable:
            return False, _("Order Rejected: no enough stock {order_book_id} to sell, you want to sell {quantity}, sellable {sellable}").format(
                order_book_id=order_book_id,
                quantity=abs(order.quantity),
                sellable=position.sellable,
            )

        # check whether is trading
        # FIXME need a better solution
        if bar.volume == 0:
            return False, _("Order Rejected: {order_book_id} is not trading.").format(
                order_book_id=order_book_id,
            )

        # # TODO check whether is limit up or limit down
        # # FIXME need to handle ST 5%
        # last_close = self.data_proxy.history(order_book_id, 2, "1d", "close").iloc[-2]
        # if is_buy and price >= last_close * 1.1:
        #     return False, _("Order Rejected: {order_book_id} is limit up.").format(
        #         order_book_id=order_book_id,
        #         )
        # elif not is_buy and price <= last_close * 0.9:
        #     return False, _("Order Rejected: {order_book_id} is limit down.").format(
        #         order_book_id=order_book_id,
        #         )

        # TODO check volume is over 25%
        # FIXME might have mulitiple order
        if amount > bar.volume * 0.25:
            return False, _("Order Rejected: {order_book_id} volume is over 25%.").format(
                order_book_id=order_book_id,
            )

        return True, None

    def handle_dividend_ex_dividend(self):
        data_proxy = self.data_proxy
        portfolio = self.account.portfolio

        for order_book_id, position in iteritems(portfolio.positions):
            dividend_series = data_proxy.get_dividends_by_book_date(order_book_id, self.current_date)
            if dividend_series is None:
                continue
            dividend_per_share = dividend_series["dividend_cash_before_tax"] / dividend_series["round_lot"]
            portfolio._dividend_info[order_book_id] = Dividend(order_book_id, position.quantity, dividend_series)
            portfolio.dividend_receivable += dividend_per_share * position.quantity

    def handle_dividend_payable(self):
        """handle dividend payable before trading
        """
        data_proxy = self.data_proxy
        portfolio = self.account.portfolio

        to_delete_dividend = []
        for order_book_id, dividend_info in iteritems(portfolio._dividend_info):
            dividend_series = dividend_info.dividend_series

            if pd.Timestamp(self.current_date) == pd.Timestamp(dividend_series.payable_date):
                dividend_per_share = dividend_series["dividend_cash_before_tax"] / dividend_series["round_lot"]
                if dividend_per_share > 0 and dividend_info.quantity > 0:
                    dividend_cash = dividend_per_share * dividend_info.quantity
                    portfolio.dividend_receivable -= dividend_cash
                    portfolio.cash += dividend_cash
                    user_log.info(_("get dividend {dividend} for {order_book_id}").format(
                        dividend=dividend_cash,
                        order_book_id=order_book_id,
                    ))
                    to_delete_dividend.append(order_book_id)

        for order_book_id in to_delete_dividend:
            portfolio._dividend_info.pop(order_book_id, None)



# -*- coding: utf-8 -*-

import codecs
import sys
import os
import shutil
import datetime
import tempfile
import tarfile

import pandas as pd
import click
import requests
from six import exec_, print_

from . import StrategyExecutor
from . import api
from .data import LocalDataProxy
from .logger import user_log, user_print
from .trading_params import TradingParams
from .utils.click_helper import Date
from .utils import dummy_func
from .scheduler import Scheduler


@click.group()
@click.option('-v', '--verbose', count=True)
@click.pass_context
def cli(ctx, verbose):
    ctx.obj["VERBOSE"] = verbose


def entry_point():
    cli(obj={})


@cli.command()
@click.option('-d', '--data-bundle-path', default=os.path.expanduser("~/.rqalpha"), type=click.Path())
def update_bundle(data_bundle_path):
    """update data bundle, download if not found"""
    day = datetime.date.today()
    tmp = os.path.join(tempfile.gettempdir(), 'rq.bundle')

    while True:
        url = 'http://7xjci3.com1.z0.glb.clouddn.com/bundles/rqbundle_%04d%02d%02d.tar.bz2' % (day.year, day.month, day.day)
        print_('try {} ...'.format(url))
        r = requests.get(url, stream=True)
        if r.status_code != 200:
            day = day - datetime.timedelta(days=1)
            continue

        out = open(tmp, 'wb')
        total_length = int(r.headers.get('content-length'))

        with click.progressbar(length=total_length, label='downloading ...') as bar:
            for data in r.iter_content(chunk_size=8192):
                bar.update(len(data))
                out.write(data)

        out.close()
        break

    shutil.rmtree(data_bundle_path, ignore_errors=True)
    os.mkdir(data_bundle_path)
    tar = tarfile.open(tmp, 'r:bz2')
    tar.extractall(data_bundle_path)
    os.remove(tmp)


@cli.command()
@click.option('-f', '--strategy-file', type=click.Path(exists=True), required=True)
@click.option('-s', '--start-date', type=Date(), required=True)
@click.option('-e', '--end-date', type=Date(), required=True)
@click.option('-o', '--output-file', type=click.Path(writable=True))
@click.option('-i', '--init-cash', default=100000, type=click.INT)
@click.option('--draw-result/--no-draw-result', default=os.name != "nt")
@click.option('--show-progress/--no-show-progress', default=True)
@click.option('-d', '--data-bundle-path', default=os.path.expanduser("~/.rqalpha"), type=click.Path())
def run(strategy_file, start_date, end_date, output_file, draw_result, data_bundle_path, init_cash, show_progress):
    '''run strategy from file
    '''
    if not os.path.exists(data_bundle_path):
        print_("data bundle not found. Run `%s update_bundle` to download data bundle." % sys.argv[0])
        return

    with codecs.open(strategy_file, encoding="utf-8") as f:
        source_code = f.read()

    results_df = run_strategy(source_code, strategy_file, start_date, end_date,
                              init_cash, data_bundle_path, show_progress)

    if output_file is not None:
        results_df.to_pickle(output_file)

    if draw_result:
        show_draw_result(strategy_file, results_df)


@cli.command()
@click.option('-d', '--directory', default="./", type=click.Path(), required=True)
def generate_examples(directory):
    '''generate example strategies to target folder
    '''
    source_dir = os.path.join(os.path.dirname(os.path.realpath(__file__)), "examples")

    try:
        shutil.copytree(source_dir, os.path.join(directory, "examples"))
    except FileExistsError:
        print("Folder examples is exists.")


@cli.command()
@click.option('-f', '--result-file', type=click.Path(exists=True), required=True)
def draw_result_df(result_file):
    '''draw result DataFrame
    '''
    results_df = pd.read_pickle(result_file)
    show_draw_result(result_file, results_df)


def run_strategy(source_code, strategy_filename, start_date, end_date, init_cash, data_bundle_path, show_progress):
    scope = {
        "logger": user_log,
        "print": user_print,
    }
    scope.update({export_name: getattr(api, export_name) for export_name in api.__all__})
    code = compile(source_code, strategy_filename, 'exec')
    exec_(code, scope)

    try:
        data_proxy = LocalDataProxy(data_bundle_path)
    except FileNotFoundError:
        print_("data bundle might crash. Run `%s update_bundle` to redownload data bundle." % sys.argv[0])
        sys.exit()

    trading_cal = data_proxy.get_trading_dates(start_date, end_date)
    Scheduler.set_trading_dates(data_proxy.get_trading_dates(start_date, datetime.date.today()))
    trading_params = TradingParams(trading_cal, start_date=start_date.date(), end_date=end_date.date(),
                                   init_cash=init_cash, show_progress=show_progress)

    executor = StrategyExecutor(
        init=scope.get("init", dummy_func),
        before_trading=scope.get("before_trading", dummy_func),
        handle_bar=scope.get("handle_bar", dummy_func),

        trading_params=trading_params,
        data_proxy=data_proxy,
    )

    results_df = executor.execute()

    return results_df


def show_draw_result(title, results_df):
    import matplotlib
    from matplotlib import gridspec
    import matplotlib.image as mpimg
    import matplotlib.pyplot as plt
    plt.style.use('ggplot')

    red = "#aa4643"
    blue = "#4572a7"
    black = "#000000"

    figsize = (18, 6)
    f = plt.figure(title, figsize=figsize)
    gs = gridspec.GridSpec(10, 8)

    # draw logo
    ax = plt.subplot(gs[:3, -1:])
    ax.axis("off")
    filename = os.path.join(os.path.dirname(os.path.realpath(__file__)), "resource")
    filename = os.path.join(filename, "ricequant-logo.png")
    img = mpimg.imread(filename)
    imgplot = ax.imshow(img, interpolation="nearest")
    ax.autoscale_view()

    # draw risk and portfolio
    series = results_df.iloc[-1]

    font_size = 12
    value_font_size = 11
    label_height, value_height = 0.8, 0.6
    label_height2, value_height2 = 0.35, 0.15

    fig_data = [
        (0.00, label_height, value_height, "Total Returns", "{0:.3%}".format(series.total_returns), red, black),
        (0.15, label_height, value_height, "Annual Returns", "{0:.3%}".format(series.annualized_returns), red, black),
        (0.00, label_height2, value_height2, "Benchmark Total", "{0:.3%}".format(series.benchmark_total_returns), blue, black),
        (0.15, label_height2, value_height2, "Benchmark Annual", "{0:.3%}".format(series.benchmark_annualized_returns), blue, black),

        (0.30, label_height, value_height, "Alpha", "{0:.4}".format(series.alpha), black, black),
        (0.40, label_height, value_height, "Beta", "{0:.4}".format(series.beta), black, black),
        (0.55, label_height, value_height, "Sharpe", "{0:.4}".format(series.sharpe), black, black),
        (0.70, label_height, value_height, "Sortino", "{0:.4}".format(series.sortino), black, black),
        (0.85, label_height, value_height, "Information Ratio", "{0:.4}".format(series.information_rate), black, black),

        (0.30, label_height2, value_height2, "Volatility", "{0:.4}".format(series.volatility), black, black),
        (0.40, label_height2, value_height2, "MaxDrawdown", "{0:.3%}".format(series.max_drawdown), black, black),
        (0.55, label_height2, value_height2, "Tracking Error", "{0:.4}".format(series.tracking_error), black, black),
        (0.70, label_height2, value_height2, "Downside Risk", "{0:.4}".format(series.downside_risk), black, black),
    ]

    ax = plt.subplot(gs[:3, :-1])
    ax.axis("off")
    for x, y1, y2, label, value, label_color, value_color in fig_data:
        ax.text(x, y1, label, color=label_color, fontsize=font_size)
        ax.text(x, y2, value, color=value_color, fontsize=value_font_size)

    # strategy vs benchmark
    ax = plt.subplot(gs[4:, :])

    ax.get_xaxis().set_minor_locator(matplotlib.ticker.AutoMinorLocator())
    ax.get_yaxis().set_minor_locator(matplotlib.ticker.AutoMinorLocator())
    ax.grid(b=True, which='minor', linewidth=.2)
    ax.grid(b=True, which='major', linewidth=1)

    ax.plot(results_df["total_returns"], label="strategy", alpha=1, linewidth=2, color=red)
    ax.plot(results_df["benchmark_total_returns"], label="benchmark", alpha=1, linewidth=2, color=blue)

    # manipulate
    vals = ax.get_yticks()
    ax.set_yticklabels(['{:3.2f}%'.format(x*100) for x in vals])

    leg = plt.legend(loc="upper left")
    leg.get_frame().set_alpha(0.5)

    plt.show()


if __name__ == '__main__':
    entry_point()



# -*- coding: utf-8 -*-


class BarObject(object):
    def __init__(self, instrument, series):
        self._series = series
        self._instrument = instrument

    @property
    def open(self):
        return self._series.open

    @property
    def close(self):
        return self._series.close

    @property
    def low(self):
        return self._series.low

    @property
    def high(self):
        return self.series.high

    @property
    def last(self):
        return self.close

    @property
    def volume(self):
        return self._series.volume

    @property
    def datetime(self):
        return self._series.name

    @property
    def instrument(self):
        return self._instrument

    @property
    def order_book_id(self):
        return self._instrument.order_book_id

    @property
    def symbol(self):
        return self._instrument.symbol

    @property
    def is_trading(self):
        return self._series.volumn > 0

    def mavg(self, intervals, frequency="day"):
        """
        Returns moving average price for the given security for a give number
            of intervals for a frequency, by default to `"day"`.
        :param int intervals: a given number of intervals, e.g. given number
            of days
        :param str frequency: frequency of the give number of intervals, by
            default as ‘day’.
        """
        raise NotImplementedError

    def vwap(self, intervals, frequency="day"):
        raise NotImplementedError

    def history(self, bar_count, frequency, field):
        raise NotImplementedError

    def __repr__(self):
        return "BarObject({0})".format(self.__dict__)

    def __getitem__(self, key):
        return self.__dict__[key]


class BarMap(object):
    def __init__(self, dt, universe, data_proxy):
        self.dt = dt
        self.universe = universe
        self.data_proxy = data_proxy

    def update_dt(self, dt):
        self.dt = dt

    def __getitem__(self, key):
        return self.data_proxy.get_bar(key, self.dt)

    def __repr__(self):
        return "{0}()".format(type(self).__name__)



# -*- coding: utf-8 -*-

import uuid

from enum import Enum

from ..const import ORDER_STATUS


def gen_order_id():
    return uuid.uuid4().hex


# TODO use nametuple to reduce memory

class Order(object):

    def __init__(self, dt, order_book_id, quantity, style):
        self.dt = dt
        self.order_book_id = order_book_id
        self._style = style
        self._order_id = gen_order_id()

        self.filled_shares = 0.0
        self.quantity = quantity
        self._reject_reason = ""

        self.status = ORDER_STATUS.OPEN

    @property
    def style(self):
        return self._style

    @property
    def order_id(self):
        return self._order_id

    @property
    def instrument(self):
        raise NotImplementedError

    def cancel(self):
        raise NotImplementedError

    def fill(self, shares):
        self.filled_shares += shares

        assert self.filled_shares <= self.quantity

    def mark_rejected(self, reject_reason):
        self._reject_reason = reject_reason
        self.status = ORDER_STATUS.REJECTED

    @property
    def is_buy(self):
        return self.quantity > 0

    @property
    def reject_reason(self):
        return self._reject_reason

    def __repr__(self):
        return "Order({0})".format(self.__dict__)



# -*- coding: utf-8 -*-


# TODO make field readonly
# TODO use nametuple to reduce memory


class Risk(object):
    def __init__(self):
        self.volatility = .0
        self.alpha = .0
        self.beta = .0
        self.sharpe = .0
        self.sortino = .0
        self.information_rate = .0
        self.max_drawdown = .0
        self.tracking_error = .0
        self.downside_risk = .0

    def __repr__(self):
        return "Risk({0})".format(self.__dict__)



# -*- coding: utf-8 -*-
import abc

from six import with_metaclass, iteritems


class BaseTax(with_metaclass(abc.ABCMeta)):
    @abc.abstractmethod
    def get_tax(self, data_proxy, order):
        raise NotImplementedError


class AStockTax(BaseTax):
    def __init__(self, tax_rate=0.001):
        self.tax_rate = tax_rate

    def get_tax(self, order, trade):
        cost_money = trade.price * abs(trade.amount)
        tax = 0
        if trade.amount < 0:
            tax = cost_money * self.tax_rate

        return tax



# -*- coding: utf-8 -*-
import abc

from six import with_metaclass, iteritems


class BaseSlippageDecider(with_metaclass(abc.ABCMeta)):
    @abc.abstractmethod
    def get_trade_price(self, data_proxy, order):
        raise NotImplementedError


class FixedPercentSlippageDecider(BaseSlippageDecider):
    def __init__(self, rate=0.246):
        self.rate = rate / 100.

    def get_trade_price(self, data_proxy, order):
        bar = data_proxy.latest_bar(order.order_book_id)

        slippage = bar.close * self.rate / 2 * (1 if order.is_buy else -1)
        trade_price = bar.close + slippage

        return trade_price



# -*- coding: utf-8 -*-

import pandas as pd

from . import ExecutionContext


class HybridDataFrame(pd.DataFrame):

    def __init__(self, *args, **kwargs):
        missing_handler = kwargs.pop("missing_handler")
        super(HybridDataFrame, self).__init__(*args, **kwargs)
        self.missing_handler = missing_handler

    def __getitem__(self, key):
        try:
            return super(HybridDataFrame, self).__getitem__(key)
        except KeyError as e:
            if not isinstance(key, str) or self.missing_handler is None:
                raise
            try:
                rv = self.missing_handler(key)
            except KeyError:
                raise e
            self[key] = rv
            return rv


def missing_handler(key, bar_count, frequency, field):
    order_book_id = key
    data_proxy = ExecutionContext.get_strategy_executor().data_proxy
    hist = data_proxy.history(order_book_id, bar_count, frequency, field)
    series = hist

    executor = ExecutionContext.get_strategy_executor()
    executor.current_universe.add(key)

    return series



# -*- coding: utf-8 -*-

from __future__ import division
from functools import partial
from collections import Iterable
import copy

import six

from .instruments import Instrument
from .logger import user_log
from .utils import ExecutionContext
from .utils.history import HybridDataFrame, missing_handler
from .i18n import gettext as _
from .scheduler import scheduler
from .const import EXECUTION_PHASE


__all__ = [
    'scheduler'
]


def export_as_api(func):
    __all__.append(func.__name__)
    return func


@export_as_api
@ExecutionContext.enforce_phase(EXECUTION_PHASE.HANDLE_BAR,
                                EXECUTION_PHASE.SCHEDULED)
def order_shares(id_or_ins, amount, style=None):
    """
    Place an order by specified number of shares. Order type is also
        passed in as parameters if needed. If style is omitted, it fires a
        market order by default.
    :PARAM id_or_ins: the instrument to be ordered
    :type id_or_ins: str or Instrument
    :param float amount: Number of shares to order. Positive means buy,
        negative means sell. It will be rounded down to the closest
        integral multiple of the lot size
    :param style: Order type and default is `MarketOrder()`. The
        available order types are: `MarketOrder()` and
        `LimitOrder(limit_price)`
    :return:  A unique order id.
    :rtype: int
    """
    if isinstance(id_or_ins, Instrument):
        order_book_id = id_or_ins.order_book_id
    else:
        order_book_id = id_or_ins

    if amount == 0:
        user_log.error(_("order_shares {order_book_id} amount is 0.").format(
            order_book_id=order_book_id,
        ))
        return

    amount = int(amount) // 100 * 100

    if not isinstance(id_or_ins, six.string_types):
        raise NotImplementedError

    bar_dict = ExecutionContext.get_current_bar_dict()
    order = get_simu_exchange().create_order(bar_dict, order_book_id, amount, style)

    return order.order_id


@export_as_api
@ExecutionContext.enforce_phase(EXECUTION_PHASE.HANDLE_BAR,
                                EXECUTION_PHASE.SCHEDULED)
def order_lots(id_or_ins, amount, style=None):
    """
    Place an order by specified number of lots. Order type is also passed
        in as parameters if needed. If style is omitted, it fires a market
        order by default.
    :param id_or_ins: the instrument to be ordered
    :type id_or_ins: str or Instrument
    :param float amount: Number of lots to order. Positive means buy,
        negative means sell.
    :param style: Order type and default is `MarketOrder()`. The
        available order types are: `MarketOrder()` and
        `LimitOrder(limit_price)`
    :return:  A unique order id.
    :rtype: int
    """
    return order_shares(id_or_ins, amount * 100, style)


@export_as_api
@ExecutionContext.enforce_phase(EXECUTION_PHASE.HANDLE_BAR,
                                EXECUTION_PHASE.SCHEDULED)
def order_value(id_or_ins, cash_amount, style=None):
    """
    Place ann order by specified value amount rather than specific number
        of shares/lots. Negative cash_amount results in selling the given
        amount of value, if the cash_amount is larger than you current
        security’s position, then it will sell all shares of this security.
        Orders are always truncated to whole lot shares.
    :param id_or_ins: the instrument to be ordered
    :type id_or_ins: str or Instrument
    :param float cash_amount: Cash amount to buy / sell the given value of
        securities. Positive means buy, negative means sell.
    :param style: Order type and default is `MarketOrder()`. The
        available order types are: `MarketOrder()` and
        `LimitOrder(limit_price)`
    :return:  A unique order id.
    :rtype: int
    """
    if isinstance(id_or_ins, Instrument):
        order_book_id = id_or_ins.order_book_id
    else:
        order_book_id = id_or_ins

    # TODO market order might be different
    bar_dict = ExecutionContext.get_current_bar_dict()
    price = bar_dict[order_book_id].close
    amount = ((cash_amount // price) // 100) * 100

    # if the cash_amount is larger than you current security’s position,
    # then it will sell all shares of this security.
    position = get_simu_exchange().account.portfolio.positions[order_book_id]
    if amount < 0:
        if abs(amount) > position.sellable:
            amount = -position.sellable

    return order_shares(id_or_ins, amount, style)


@export_as_api
@ExecutionContext.enforce_phase(EXECUTION_PHASE.HANDLE_BAR,
                                EXECUTION_PHASE.SCHEDULED)
def order_percent(id_or_ins, percent, style=None):
    """
    Place an order for a security for a given percent of the current
        portfolio value, which is the sum of the positions value and
        ending cash balance. A negative percent order will result in
        selling given percent of current portfolio value. Orders are
        always truncated to whole shares. Percent should be a decimal
        number (0.50 means 50%), and its absolute value is <= 1.
    :param id_or_ins: the instrument to be ordered
    :type id_or_ins: str or Instrument
    :param float percent: Percent of the current portfolio value. Positive
        means buy, negative means selling give percent of the current
        portfolio value. Orders are always truncated according to lot size.
    :param style: Order type and default is `MarketOrder()`. The
        available order types are: `MarketOrder()` and
        `LimitOrder(limit_price)`
    :return:  A unique order id.
    :rtype: int
    """
    assert 0 < percent <= 1

    portfolio_value = get_simu_exchange().account.portfolio.portfolio_value
    return order_value(id_or_ins, portfolio_value * percent, style)


@export_as_api
@ExecutionContext.enforce_phase(EXECUTION_PHASE.HANDLE_BAR,
                                EXECUTION_PHASE.SCHEDULED)
def order_target_value(id_or_ins, cash_amount, style=None):
    """
    Place an order to adjust a position to a target value. If there is no
        position for the security, an order is placed for the whole amount
        of target value. If there is already a position for the security,
        an order is placed for the difference between target value and
        current position value.
    :param id_or_ins: the instrument to be ordered
    :type id_or_ins: str or Instrument
    :param float cash_amount: Target cash value for the adjusted position
        after placing order.
    :param style: Order type and default is `MarketOrder()`. The
        available order types are: `MarketOrder()` and
        `LimitOrder(limit_price)`
    :return:  A unique order id.
    :rtype: int
    """
    if isinstance(id_or_ins, Instrument):
        order_book_id = id_or_ins.order_book_id
    else:
        order_book_id = id_or_ins

    # TODO market order might be different
    bar_dict = ExecutionContext.get_current_bar_dict()
    price = bar_dict[order_book_id].close
    position = get_simu_exchange().account.portfolio.positions[order_book_id]

    current_value = position.quantity * price

    return order_value(order_book_id, cash_amount - current_value, style)


@export_as_api
@ExecutionContext.enforce_phase(EXECUTION_PHASE.HANDLE_BAR,
                                EXECUTION_PHASE.SCHEDULED)
def order_target_percent(id_or_ins, percent, style=None):
    """
    Place an order to adjust position to a target percent of the portfolio
        value, so that your final position value takes the percentage you
        defined of your whole portfolio.
        position_to_adjust = target_position - current_position
        Portfolio value is calculated as sum of positions value and ending
        cash balance. The order quantity will be rounded down to integral
        multiple of lot size. Percent should be a decimal number (0.50
        means 50%), and its absolute value is <= 1. If the
        position_to_adjust calculated is positive, then it fires buy
        orders, otherwise it fires sell orders.
    :param id_or_ins: the instrument to be ordered
    :type id_or_ins: str or Instrument
    :param float percent: Number of percent to order. It will be rounded down
        to the closest integral multiple of the lot size
    :param style: Order type and default is `MarketOrder()`. The
        available order types are: `MarketOrder()` and
        `LimitOrder(limit_price)`
    :return:  A unique order id.
    :rtype: int
    """
    if isinstance(id_or_ins, Instrument):
        order_book_id = id_or_ins.order_book_id
    else:
        order_book_id = id_or_ins

    # TODO market order might be different
    bar_dict = ExecutionContext.get_current_bar_dict()
    price = bar_dict[order_book_id].close
    position = get_simu_exchange().account.portfolio.positions[order_book_id]

    current_value = position.quantity * price
    portfolio_value = get_simu_exchange().account.portfolio.portfolio_value

    return order_value(order_book_id, portfolio_value * percent - current_value, style)


@export_as_api
@ExecutionContext.enforce_phase(EXECUTION_PHASE.BEFORE_TRADING,
                                EXECUTION_PHASE.HANDLE_BAR,
                                EXECUTION_PHASE.SCHEDULED)
def get_order(order_id):
    """
    Get a specified order by the unique order_id. The order object will be
        discarded at end of handle_bar.
    :param int order_id: order’s unique identifier returned by function
        like `order_shares`
    :return: an `Order` object.
    """
    return get_simu_exchange().get_order(order_id)


@export_as_api
@ExecutionContext.enforce_phase(EXECUTION_PHASE.BEFORE_TRADING,
                                EXECUTION_PHASE.HANDLE_BAR,
                                EXECUTION_PHASE.SCHEDULED)
def get_open_orders():
    return copy.deepcopy(get_simu_exchange().open_orders)


@export_as_api
@ExecutionContext.enforce_phase(EXECUTION_PHASE.BEFORE_TRADING,
                                EXECUTION_PHASE.HANDLE_BAR,
                                EXECUTION_PHASE.SCHEDULED)
def cancel_order(order_id):
    get_simu_exchange().cancel_order(order_id)


@export_as_api
def update_universe(id_or_symbols):
    """
    This method takes one or a list of id_or_symbol(s) as argument(s), to
        update the current subscription set of the instruments. It takes
        effect on the next bar event.
    :param id_or_symbols: one or a list of id_or_symbol(s).
    :type id_or_symbols: str or an iterable of strings
    """
    if isinstance(id_or_symbols, six.string_types):
        id_or_symbols = [id_or_symbols]
    elif isinstance(id_or_symbols, Instrument):
        id_or_symbols = [Instrument.order_book_id]
    elif isinstance(id_or_symbols, Iterable):
        id_or_symbols = [
            (item.order_book_id if isinstance(item, Instrument) else item)
            for item in id_or_symbols
        ]
    else:
        raise RuntimeError(_("unknown type"))

    executor = get_strategy_executor()
    executor.current_universe = set(id_or_symbols)


@export_as_api
def instruments(id_or_symbols):
    """
    Convert a string or a list of strings as order_book_id to instrument
        object(s).
    :param id_or_symbols: Passed in strings / iterable of strings are
        interpreted as order_book_ids. China market’s order_book_ids are
        like ‘000001.XSHE’ while US’s market’s order_book_ids are like
        ‘AAPL.US’
    :type id_or_symbols: str or iterable of strings
    :return: one / a list of instrument(s) object(s) - by the
        id_or_symbol(s) requested.
    """
    if isinstance(id_or_symbols, six.string_types):
        return get_data_proxy().instrument(id_or_symbols)

    return map(get_data_proxy().instrument, id_or_symbols)


@export_as_api
@ExecutionContext.enforce_phase(EXECUTION_PHASE.BEFORE_TRADING,
                                EXECUTION_PHASE.HANDLE_BAR,
                                EXECUTION_PHASE.SCHEDULED)
def history(bar_count, frequency, field):
    executor = get_strategy_executor()
    data_proxy = get_data_proxy()

    results = {}

    # This make history slow
    # for order_book_id in list(executor.current_universe)[:1]:
    #     hist = data_proxy.history(order_book_id, bar_count, frequency, field)
    #     results[order_book_id] = hist

    handler = partial(missing_handler, bar_count=bar_count, frequency=frequency, field=field)
    return HybridDataFrame(results, missing_handler=handler)


@export_as_api
def is_st_stock(order_book_id):
    """
    instrument, = _get_instruments([order_book_id])
    return instrument.is_st
    """
    raise NotImplementedError


@export_as_api
@ExecutionContext.enforce_phase(EXECUTION_PHASE.HANDLE_BAR,
                                EXECUTION_PHASE.SCHEDULED)
def plot(series_name, value):
    """
    Add a point to custom series.
    :param str series_name: the name of custom series
    :param float value: the value of the series in this time
    :return: None
    """


def get_simu_exchange():
    return ExecutionContext.get_exchange()


def get_strategy_context():
    return ExecutionContext.get_strategy_context()


def get_strategy_executor():
    return ExecutionContext.get_strategy_executor()


def get_current_dt():
    return ExecutionContext.get_current_dt()


def get_data_proxy():
    return ExecutionContext.get_strategy_executor().data_proxy



# -*- coding: utf-8 -*-

# TODO use nametuple to reduce memory


class Trade(object):

    def __init__(self, date, order_book_id, price, amount, order_id, commission=0., tax=0.):
        self.date = date
        self.order_book_id = order_book_id
        self.price = price
        self.amount = amount
        self.order_id = order_id
        self.commission = commission
        self.tax = tax

    def __repr__(self):
        return "Trade({0})".format(self.__dict__)



import pytz
import six

import pandas as pd
from ..instruments import Instrument


class LocalDataSource:
    DAILY = 'daily.bcolz'
    INSTRUMENTS = 'instruments.pk'
    DIVIDEND = 'dividend.bcolz'
    TRADING_DATES = 'trading_dates.bcolz'
    YIELD_CURVE = 'yield_curve.bcolz'

    YIELD_CURVE_TENORS = {
        0: 'S0',
        30: 'M1',
        60: 'M2',
        90: 'M3',
        180: 'M6',
        270: 'M9',
        365: 'Y1',
        365 * 2: 'Y2',
        365 * 3: 'Y3',
        365 * 4: 'Y4',
        365 * 5: 'Y5',
        365 * 6: 'Y6',
        365 * 7: 'Y7',
        365 * 8: 'Y8',
        365 * 9: 'Y9',
        365 * 10: 'Y10',
        365 * 15: 'Y15',
        365 * 20: 'Y20',
        365 * 30: 'Y30',
        365 * 40: 'Y40',
        365 * 50: 'Y50',
    }

    YIELD_CURVE_DURATION = sorted(YIELD_CURVE_TENORS.keys())

    PRICE_SCALE = 1000.

    def __init__(self, root_dir):
        self._root_dir = root_dir
        import bcolz
        import os
        import pickle
        self._daily_table = bcolz.open(os.path.join(root_dir, LocalDataSource.DAILY))
        self._instruments = {d['order_book_id']: Instrument(d)
                             for d in pickle.load(open(os.path.join(root_dir, LocalDataSource.INSTRUMENTS), 'rb'))}
        self._dividend = bcolz.open(os.path.join(root_dir, LocalDataSource.DIVIDEND))
        self._yield_curve = bcolz.open(os.path.join(root_dir, LocalDataSource.YIELD_CURVE))
        self._trading_dates = pd.Index(pd.Timestamp(str(d), tz=pytz.utc) for d in
                                       bcolz.open(os.path.join(root_dir, LocalDataSource.TRADING_DATES)))

    def instruments(self, order_book_ids):
        if isinstance(order_book_ids, six.string_types):
            try:
                return self._instruments[order_book_ids]
            except KeyError:
                print('ERROR: order_book_id {} not exists!'.format(order_book_ids))
                return None

        return [self._instruments[ob] for ob in order_book_ids
                if ob in self._instruments]

    def all_instruments(self, itype='CS'):
        if itype is None:
            return pd.DataFrame([[v.order_book_id, v.symbol, v.abbrev_symbol, v.type]
                                 for v in self._instruments.values()],
                                columns=['order_book_id', 'symbol', 'abbrev_symbol', 'type'])

        if itype not in ['CS', 'ETF', 'LOF', 'FenjiA', 'FenjiB', 'FenjiMu', 'INDX', 'Future']:
            raise ValueError('Unknown type {}'.format(itype))

        return pd.DataFrame([v.__dict__ for v in self._instruments.values() if v.type == itype])

    def sector(self, code):
        return [v.order_book_id for v in self._instruments.values()
                if v.type == 'CS' and v.sector_code == code]

    def industry(self, code):
        return [v.order_book_id for v in self._instruments.values()
                if v.type == 'CS' and v.industry_code == code]

    def concept(self, *concepts):
        return [v.order_book_id for v in self._instruments.values()
                if v.type == 'CS' and any(c in v.concept_names.split('|') for c in concepts)]

    def get_trading_dates(self, start_date, end_date):
        left = self._trading_dates.searchsorted(start_date)
        right = self._trading_dates.searchsorted(end_date, side='right')
        return self._trading_dates[left:right]

    def get_yield_curve(self, start_date, end_date):
        duration = (end_date - start_date).days
        tenor = 0
        for t in LocalDataSource.YIELD_CURVE_DURATION:
            if duration >= t:
                tenor = t
            else:
                break

        d = start_date.year * 10000 + start_date.month * 100 + start_date.day
        return self._yield_curve.fetchwhere('date<={}'.format(d)).cols[self.YIELD_CURVE_TENORS[tenor]][-1] / 10000.0

    def get_dividends(self, order_book_id):
        try:
            sid = self._dividend.attrs['stock_id'][order_book_id]
        except KeyError:
            return pd.DataFrame()

        dividends = self._dividend.fetchwhere('id=={}'.format(sid))
        return pd.DataFrame({
            'book_closure_date': pd.Index(pd.Timestamp(str(d)) for d in dividends.cols['closure_date']),
            'ex_dividend_date': pd.Index(pd.Timestamp(str(d)) for d in dividends.cols['ex_date']),
            'payable_date': pd.Index(pd.Timestamp(str(d)) for d in dividends.cols['payable_date']),
            'dividend_cash_before_tax': dividends.cols['cash_before_tax'][:] / 10000.0,
            'round_lot': dividends.cols['round_lot']
        }, index=pd.Index(pd.Timestamp(str(d)) for d in dividends.cols['announcement_date']))

    def get_all_bars(self, order_book_id):
        try:
            sid = self._daily_table.attrs['id_map'][order_book_id]
        except KeyError:
            raise RuntimeError('No data for {}'.format(order_book_id))

        bars = self._daily_table.fetchwhere('id=={}'.format(sid))
        return pd.DataFrame({
            'open': (bars.cols['open'][:] / self.PRICE_SCALE).round(2),
            'close': (bars.cols['close'][:] / self.PRICE_SCALE).round(2),
            'high': (bars.cols['high'][:] / self.PRICE_SCALE).round(2),
            'low': (bars.cols['low'][:] / self.PRICE_SCALE).round(2),
            'volume': bars.cols['volume'],
        }, index=pd.Index(pd.Timestamp(str(d)) for d in bars.cols['date']))



# -*- coding: utf-8 -*-

import pytz
from enum import Enum

from .const import EVENT_TYPE


class SimulatorAStockTradingEventSource(object):
    def __init__(self, trading_param):
        self.trading_param = trading_param
        self.timezone = trading_param.timezone
        self.generator = self.create_generator()

    def create_generator(self):
        for date in self.trading_param.trading_calendar:
            yield date.replace(hour=9, minute=0, tzinfo=self.timezone), EVENT_TYPE.DAY_START
            yield date.replace(hour=15, minute=0, tzinfo=self.timezone), EVENT_TYPE.HANDLE_BAR
            yield date.replace(hour=16, minute=0, tzinfo=self.timezone), EVENT_TYPE.DAY_END

    def __iter__(self):
        return self

    def __next__(self):
        for date, event in self.generator:
            return date, event

        raise StopIteration

    next = __next__  # Python 2



#!/usr/bin/env python

from setuptools import setup, find_packages

from pip.req import parse_requirements


setup(
    name='rqalpha',
    version='0.0.55',
    description='Ricequant Backtest Engine',
    packages=find_packages(exclude=[]),
    author='ricequant',
    author_email='public@ricequant.com',
    package_data={'': ['*.*']},
    url='https://github.com/ricequant/rqalpha',
    install_requires=[str(ir.req) for ir in parse_requirements("requirements.txt", session=False)],
    zip_safe=False,
    entry_points={
        "console_scripts": [
            "rqalpha = rqalpha.__main__:entry_point",
        ]
    },
)



# -*- coding: utf-8 -*-

from pprint import pprint

from six import iteritems

from rqalpha import StrategyExecutor
from rqalpha.api import *
from .fixture import *


def test_strategy_print_call(trading_params, data_proxy):

    def init(context):
        print("init", context)

    def before_trading(context, bar_dict):
        print("before_trading", context.now)

    def handle_bar(context, bar_dict):
        print(bar_dict["000001.XSHE"])
        print(bar_dict["600485.XSHG"])
        print(bar_dict["600099.XSHG"])
        print(context.now)

    executor = StrategyExecutor(
        init=init,
        before_trading=before_trading,
        handle_bar=handle_bar,

        trading_params=trading_params,
        data_proxy=data_proxy,
    )

    perf = executor.execute()


def test_strategy_load_data(trading_params, data_proxy):

    def init(context):
        print("init", context)
        context.stock = "600099.XSHG"
        context.close = []

    def before_trading(context, bar_dict):
        pass

    def handle_bar(context, bar_dict):
        context.close.append(bar_dict[context.stock].close)

    executor = StrategyExecutor(
        init=init,
        before_trading=before_trading,
        handle_bar=handle_bar,

        trading_params=trading_params,
        data_proxy=data_proxy,
    )

    perf = executor.execute()

    assert len(executor.strategy_context.close) == len(trading_params.trading_calendar)
    assert executor.strategy_context.close[-1] == data_proxy.get_bar(executor.strategy_context.stock,
                                                                     trading_params.trading_calendar[-1]).close


def test_strategy_portfolio(trading_params, data_proxy):

    def handle_bar(context, bar_dict):
        pass
        # print(context.portfolio)

    executor = StrategyExecutor(
        handle_bar=handle_bar,

        trading_params=trading_params,
        data_proxy=data_proxy,
    )

    perf = executor.execute()


def test_strategy_order(trading_params, data_proxy):

    def handle_bar(context, bar_dict):
        order_shares("000001.XSHE", 100)

    executor = StrategyExecutor(
        handle_bar=handle_bar,

        trading_params=trading_params,
        data_proxy=data_proxy,
    )

    perf = executor.execute()


def test_strategy_keep_buy(trading_params, data_proxy):

    def handle_bar(context, bar_dict):
        order_shares("000001.XSHE", 1000)
        print(context.now)
        print(context.portfolio.positions["000001.XSHE"])
        print(context.portfolio.cash, context.portfolio.market_value, context.portfolio.portfolio_value)

    executor = StrategyExecutor(
        handle_bar=handle_bar,

        trading_params=trading_params,
        data_proxy=data_proxy,
    )

    perf = executor.execute()


def test_strategy_buy_and_sell(trading_params, data_proxy):
    def init(context):
        context.cnt = 0

    def handle_bar(context, bar_dict):
        context.cnt += 1

        if context.cnt == 2:
            order_shares("000001.XSHE", 5000)
            order_shares("000001.XSHE", -5000)
        if context.cnt == 3:
            order_shares("000001.XSHE", -5000)
        if context.cnt == 4:
            order_shares("000001.XSHE", -5000)

    executor = StrategyExecutor(
        init=init,
        handle_bar=handle_bar,

        trading_params=trading_params,
        data_proxy=data_proxy,
    )

    perf = executor.execute()

    positions = executor.exchange.account.portfolio.positions
    position = positions["000001.XSHE"]
    assert position.quantity == 0
    assert position.sellable == 0


def test_strategy_buy_and_sell2(trading_params, data_proxy):
    def init(context):
        context.cnt = 0

    def handle_bar(context, bar_dict):
        context.cnt += 1

        if context.cnt % 2 == 1:
            order_shares("000001.XSHE", 5000)
        else:
            order_shares("000001.XSHE", -5000)

    executor = StrategyExecutor(
        init=init,
        handle_bar=handle_bar,

        trading_params=trading_params,
        data_proxy=data_proxy,
    )

    perf = executor.execute()

    positions = executor.exchange.account.portfolio.positions
    position = positions["000001.XSHE"]
    assert position.quantity == 0
    assert position.sellable == 0

    pprint(executor.exchange.account.get_all_trades())
    for date, portfolio in iteritems(executor.exchange.daily_portfolios):
        print(date)
        pprint(portfolio)


def test_strategy_sell_no_sellable(trading_params, data_proxy):
    def init(context):
        context.cnt = 0

    def handle_bar(context, bar_dict):
        order_shares("000001.XSHE", 5000)
        order_shares("000001.XSHE", -5000)

    executor = StrategyExecutor(
        init=init,
        handle_bar=handle_bar,

        trading_params=trading_params,
        data_proxy=data_proxy,
    )

    perf = executor.execute()


def test_strategy_history(trading_params, data_proxy):
    def init(context):
        context.cnt = 0
        update_universe(["000001.XSHE"])

    def handle_bar(context, bar_dict):
        print(history(5, "1d", "close")["000001.XSHE"])

    executor = StrategyExecutor(
        init=init,
        handle_bar=handle_bar,

        trading_params=trading_params,
        data_proxy=data_proxy,
    )

    perf = executor.execute()



# -*- coding: utf-8 -*-

from six import iteritems

from .context import ExecutionContext


def memoize(function):
    memo = {}
    function.__memo__ = memo

    def wrapper(*args, **kwargs):
        key = "#".join([str(arg) for arg in args] + ["%s:%s" % (k, v) for k, v in iteritems(kwargs)])
        if key in memo:
            return memo[key]
        else:
            rv = function(*args, **kwargs)
            memo[key] = rv
            return rv

    return wrapper


def dummy_func(*args, **kwargs):
    return None



# -*- coding: utf-8 -*-

import datetime
import copy
from collections import defaultdict, OrderedDict

from .analyser.portfolio import Portfolio
from .analyser.commission import AStockCommission, BaseCommission
from .analyser.slippage import FixedPercentSlippageDecider, BaseSlippageDecider
from .analyser.tax import AStockTax


class Account(object):
    def __init__(self, **kwargs):
        self._slippage_decider = kwargs.get("slippage", FixedPercentSlippageDecider())
        self._commission_decider = kwargs.get("commission", AStockCommission())
        self._tax_decider = kwargs.get("tax", AStockTax())

        self._daily_trades = defaultdict(list)            # type: Dict[date, List[Trade]]
        self._portfolio = Portfolio()

        # init portfolio
        self._portfolio.cash = self._portfolio.starting_cash = kwargs.get("init_cash", 100000.)

        self._portfolio.portfolio_value = self._portfolio.cash
        self._portfolio.start_date = kwargs.get("start_date")

    @property
    def slippage_decider(self):
        return self._slippage_decider

    @slippage_decider.setter
    def slippage_decider(self, value):
        assert isinstance(value, BaseSlippageDecider)
        self._slippage_decider = value

    @property
    def commission_decider(self):
        return self._commission_decider

    @commission_decider.setter
    def commission_decider(self, value):
        assert isinstance(value, BaseCommission)
        self._commission_decider = value

    @property
    def tax_decider(self):
        return self._tax_decider

    @property
    def portfolio(self):
        """account portfolio
        Do not edit the results if you don't know what are you doing.

        :returns:
        :rtype: Portfolio

        """
        return self._portfolio

    def get_trades(self, date):
        return self._daily_trades[date]

    def get_all_trades(self):
        """get all trades

        :returns:
        :rtype: Dict[date, List[]]

        """
        return self._daily_trades

    def set_start_date(self, start_date):
        assert isinstance(date, datetime.date)
        self._portfolio.start_date = start_date

    def set_init_cash(self, cash):
        assert cash > 0
        self._portfolio.cash = self._portfolio.starting_cash = cash

    def record_new_trade(self, date, trade):
        """record new trade in Account

        :param datetime.date date:
        :param Trade trade:
        :returns: None
        :rtype:

        """
        self._daily_trades[date].append(trade)

    @property
    def portfolio_value(self):
        return self._portfolio.portfolio_value

    @property
    def cash(self):
        return self._portfolio.cash
